/*********************************************************************
 * Soundlib.S
 *********************************************************************

Copyright (c) 1998 David Kuehling

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice (including the next
paragraph) shall be included in all copies or substantial portions of the
Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

 * Last Modification: 8th july 1998
 *
 *  - needs to be precompiled by the PreFargo - precompiler
 */

;;  For further details see soundlib.txt. Repating the most relevant bits here:

;; HOW LUCKY YOU HAVE A TI-92 THAT STILL WORKS TODAY.  DON'T TRY ANY OF THIS
;; UNLESS YOU CAN AFFORD IRREPARABLY BREAKING OR DAMAGING YOUR TI-92.  IF YOU
;; REALLY WANT TO GIVE IT A TRY, RUN THIS INSIDE A TI-92 EMULATOR INSTEAD.
;;
;; I TAKE NO RESPONSIBILITY FOR DAMAGE CAUSED DIRECTLY OR INDIRECTLY BY MY
;; PROGRAM, OR BY ANY PERSON, TRYING TO CONNECT AUDIO HARDWARE.  If you really
;; want to try this, proceed with utmost skill and diligence, make sure you
;; thoroughly understand the electrical characteristics and absolute maximum
;; ratings of the serial port and the audio hardware you connect.  PROCEED AT
;; YOUR OWN RISK.
;;
	XDEF	_library
	XDEF	soundlib@0000
	XDEF	soundlib@0001
	XDEF	soundlib@0002
	XDEF	soundlib@0003
	XDEF	soundlib@0004


/**********************************************************************
 * Init
 **********************************************************************
 * Input: nothing
 * Output: nothing
 *
 * Initializes a sound constant, by testing the Calculators speed
 * (bewares compatability between TI-92, 256K TI-92, TI-92 II, TI-92+ and
 * boosted calcs)
 */
soundlib@0000:
Init:
	movem.l	d0-d2/a0, -(a7)

	move.w	#$700, d0		// deactivate interrupts
	trap	#1

	lea	$600017, a0
	moveq	#$E0-256, d2		// wait, until ($600017) == $E0
\Loop1:	cmp.b	(a0), d2
	bne	\Loop1

	st.b	d2			// wait, until ($600017) == $FF
	clr.w	d1
\Loop2:	addq.w	#1, d1
	move.l	(a7), (a7)		// some delay, to compensate the Byte-Port-Access
	move.l	(a7), (a7)
	cmp.b	(a0), d2
	bne	\Loop2

	trap	#1			// restore old SR

	mulu	#267, d1		// calculate the OneSecond - Value
	lea	OneSecond(PC), a0
	move.l	d1, (a0)

	movem.l	(a7)+, d0-d2/a0
	rts

/**********************************************************************
 * Sound
 **********************************************************************
 * Input:
 *  d0.w = Frequency (should be in about the range 40...4000)
 *  d1.w = Duration (1/1024 seconds)
 * Output:
 *  Tone is played through the serial port
 */

soundlib@0001:
Sound:
	movem.l	d0-d3/d7, -(a7)

	bset	#6, $60000F		// enable direct port access

	move.l	OneSecond(PC), d3
	move.l 	d3, d2			// d2 = T/2 sec
	lsr.l	#1, d2
	divu	d0, d2
	subq.w	#4, d2			// d2 -= 4  (compensate loop-entry overhead etc.)
	ext.l	d2

	lsr.l	#8, d3
	mulu	d1, d3			// d3 = t/2 = d1 sec / 2048 (counter)
	lsr.l	#3, d3

	move.w	#$700, d0		// switch off interrupts for smoother sound
	trap	#1
\Loop:
	move.w	d2, d7			// d7 = counter
	and.b   #$FC, $60000F           // red & white wires = off
\Wait1: dbra	d7, \Wait1		// wait T/2 sec

	move.w	d2, d7			// d7 = counter
	or.b    #3, $60000F             // red & white wires = on
\Wait2: dbra	d7, \Wait2		// wait T/2 sec

	sub.l	d2, d3			// d3 -= T/2
	subq.l	#8, d3			// compensate loop-entry overhead etc.
	bpl	\Loop

	trap	#1			// restore old status register
	movem.l	(a7)+, d0-d3/d7
	rts

/**********************************************************************
 * Pause
 **********************************************************************
 * Input:
 *  d1.w = Duration (1/1024 seconds)
 * Ouput:
 *  Waits
 */
soundlib@0002:
Pause:
	movem.l	d0-d2, -(a7)
	move.l	OneSecond(PC), d2	// calculate delay-counter
	lsr.l	#8, d2
	mulu	d1, d2
	lsr.l	#2, d2
	beq	\End

	move.w	#$700, d0		// switch off interrupts
	trap	#1
\Loop:
	moveq	#127, d1		// counter
\Wait:	dbra	d1, \Wait

	moveq	#119, d1
	sub.l	d1, d2
	bpl	\Loop

	trap	#1			// restore old SR
\End:
        movem.l	(a7)+, d0-d2
	rts


/**********************************************************************
 * Play
 **********************************************************************
 * Input:
 *  a0 = Pointer to Tune data
 *  a1 = Pointer to callback routine, that is called each time before a
 *	 tone is played. if a1.l = 0, no routine will be called.
 *	 This routine is expected to don't change any register, d4
 *	 excepted. If d4.b is not zero, after the callback routine was
 *	 called, the Play routine will exit. (This can be used to abort
 *	 a tune, when a key is pressed)
 *       Contents of the registers, when it is called:
 *	   (a0).b = length of current note (a0 points to the current
 *		position within the tune-data)
 *	   d0.b = current note (0 = C0, 1 = C#0...)
 *	   d3.w = number of current note (0 = 1st, 1 = 2nd ...; can be
 *	 	  used to display song texts, according to the currently
 *	  	  played tone)
 * Ouput:
 *  Tune is played
 */
soundlib@0003
Play:
	movem.l	d0-d4/d6-d7/a0/a2, -(a7)
	lea	FrqTable(PC), a2	// a2.l = *FrqTable
	move.w	#224+1, d6		// d6.w = Note Duration+1
	move.w	#120, d7		// d7.w = Tempo
	clr.w	d3			// d3.w = note counter
	clr.b	d4			// d4.b = 0 -- don't exit
\Loop:
	move.b	(a0)+, d0		// d0 = Command
	btst.b	#7, d0			// if command >= 128: it is a controll sequence
	bne	\Controll

	/** it is a note		*/
	move.l	a1, d1			// if (a1 != NULL)
	beq	\NoCallback
	jsr	(a1)			//   Callback ();
	tst.b	d4			//  exit, if d4.b != 0
	bne	\C_tend
	addq.w	#1, d3			// increase the note counter
\NoCallback:
	ext.w	d0			// d0 = FrqTable[d0] (load frequency)
	lsl.b	#1, d0
	move.w	0(a2, d0.w), d0

	clr.w   d1			// d1 = Total Note Duration (milliseconds)
	move.b	(a0)+, d1		//    = (d*32*60 / Tempo) milliseconds
	mulu	#60*32, d1
	divu	d7, d1

	move.w	d1, d2			// d1 = Note Fill Duration; d2 = Note Pause Duration
	mulu	d6, d1
	lsr.l	#8, d1
	sub.w	d1, d2

	bsr	Sound			// play the note
	move.w	d2, d1			// pause
	bsr	Pause

	bra	\Loop

\Controll:
	/** it is a controll sequence	*/
	cmp.b	#128, d0
	beq	\C_pause
	cmp.b	#129, d0
	beq	\C_tempo
	cmp.b	#130, d0
	beq	\C_noted
	cmp.b 	#131, d0
	beq	\C_tend
	bra	\Loop

\C_pause:
	clr.w   d1			// d1 = Total Note Duration (milliseconds)
	move.b	(a0)+, d1		//    = (d*32*60 / Tempo) milliseconds
	mulu	#60*32, d1
	divu	d7, d1
	bsr	Pause
	bra	\Loop

\C_tempo:
	clr.w	d7			// d7 = new tempo
	move.b	(a0)+, d7
	bra	\Loop

\C_noted:
	clr.w	d6			// d6 = new note duration
	move.b	(a0)+, d6
	addq.w	#1, d6
	bra	\Loop

\C_tend:
	movem.l	(a7)+, d0-d4/d6-d7/a0/a2
	rts



/**********************************************************************
 * Frequency Table
 */
soundlib@0004:
FrqTable:
	/*      C    C#     D    D#     E     F    F#     G    G#     A    A#     C	       */
	dc.w   33,   35,   37,   39,   41,   44,   46,   49,   52,   55,   58,   62  /* C0-A#0 */
	dc.w   65,   69,   73,   78,   82,   87,   92,   98,  104,  110,  117,  123  /* C1-A#1 */
	dc.w  131,  139,  147,  156,  165,  175,  185,  196,  208,  220,  233,  247  /* C2-A#2 */
	dc.w  262,  277,  294,  311,  330,  349,  370,  392,  415,  440,  466,  494  /* C3-A#3 */
	dc.w  523,  554,  587,  622,  659,  698,  740,  784,  831,  880,  932,  988  /* C4-A#4 */
	dc.w 1047, 1109, 1175, 1245, 1319, 1397, 1480, 1568, 1661, 1760, 1865, 1976  /* C5-A#5 */
	dc.w 2093, 2217, 2349, 2489, 2637, 2794, 2960, 3136, 3322, 3520, 3729, 3951  /* C6-A#6 */

OneSecond:	dc.l 840000		// Loop cycles for one second

_library:	dc.b "soundlib", 0

	END
